﻿/*******************************************************************************
* Copyright 2009 Seapine Software, Inc. All rights reserved.
*
* IMPORTANT:  This Software is supplied to you by Seapine Software, Inc. (“Seapine”)
* in consideration of your agreement to the following terms, and your use, installation,
* modification or redistribution of this Software constitutes acceptance of these terms.
* If you do not agree with these terms, please do not use, install, modify or redistribute
* this Software.
*
* In consideration of your agreement to abide by the following terms, and subject to these
* terms, Seapine grants you a personal, non-exclusive license, under Seapine copyrights in
* this original Seapine software (the “Software”), to use, reproduce, modify and redistribute
* the Software, with or without modifications, in source and/or binary forms; provided that
* if you redistribute the Software in its entirety and without modifications, you must retain
* this notice and the following text and disclaimers in all such redistributions of the Software.
* Neither the name, trademarks, service marks or logos of Seapine, Inc. may be used to endorse
* or promote products derived from the Software without specific prior written permission from
* Seapine.  Except as expressly stated in this notice, no other rights or licenses, express or
* implied, are granted by Seapine herein, including but not limited to any patent rights that
* may be infringed by your derivative works or by other works in which the Software may be
* incorporated.
*
* The Software is provided by Seapine on an "AS IS" basis.  SEAPINE MAKES NO WARRANTIES,
* EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT,
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE SOFTWARE OR ITS USE AND
* OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
*
* IN NO EVENT SHALL SEAPINE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE,
* REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE SOFTWARE, HOWEVER CAUSED AND WHETHER
* UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN
* IF SEAPINE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
******************************************************************************/
using System;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;

namespace GenerateTestRuns
{
    class Program
    {
        static void Main(string[] args)
        {
            /*The program needs three parameters:
             * 
             * 1.TestTrack project name
             * 2.TestTrack filter (test case filter)
             * 3.Test Run Set
             * 
             * */
            //if wrong number of arguments, log message and exit
            if (args.Length == 3)
            {
                //Variables

                //info stored in config file
                string ttUser = "";
                string ttPassword = "";
                string ttServer = "";

                //info from parameters
                string ttProject;
                string ttFilter;
                string ttRunSet;

                //vars used
                long ttCookie = 0;//session id
                long tcNumber = 0;//test case number to generate runs for

                //Get current path to use to look for config file and create log file
                String curPath = Process.GetCurrentProcess().MainModule.FileName;
                curPath = Path.GetDirectoryName(curPath);

                //Instantiate CMConfig class and set path for config file
                CMConfig config = new CMConfig();
                config.SetPath(curPath);
                //Set path for log
                Logger.SetPath(curPath);

                //Fill in variables with values from config file
                ttUser = config.getUserNameFromFile();
                ttPassword = config.getPasswordFromFile();
                ttServer = config.getTTURLFromFile();
                if ((ttUser.Equals("")) || (ttServer.Equals("")))
                {
                    Logger.WriteLine("Information missing from config file");
                }
                else
                {
                    //Fill in variables with values from parameters
                    ttProject = args[0];
                    ttFilter = args[1];
                    ttRunSet = args[2];

                    //Instantiate SOAP object
                    ttsoapcgi ttSoap = new ttsoapcgi();
                    //Set the soap cgi URL
                    ttSoap.Url = ttServer;
                    //Create CProject object and only log in to TCM
                    CDatabase ttDatabase = new CDatabase();
                    ttDatabase.name = ttProject;
                    CProjectDataOption[] projOps = new CProjectDataOption[1];
                    projOps[0] = new CProjectDataOption();
                    projOps[0].name = "TestTrack TCM";

                    CProject ttCProject = new CProject();
                    ttCProject.database = ttDatabase;
                    ttCProject.options = projOps;


                    if (ttCProject != null)
                    {
                        try
                        {
                            //If we are able to create the CProject object, then log in
                            ttCookie = ttSoap.ProjectLogon(ttCProject, ttUser, ttPassword);
                            //Flag we'll use to check if the test run set exists
                            bool ttFlag = false;
                            //Let's get this list of test run sets and see if we need to create it
                            CFieldValue[] ttFValues = ttSoap.getDropdownFieldValuesForTable(ttCookie, "Test Run", "Test Run Set");
                            for (int i = 0; i < ttFValues.Length; i++)
                            {
                                if (ttFValues[i].value.Equals(ttRunSet))
                                {
                                    ttFlag = true;
                                }
                            }
                            if (ttFlag == false)
                            {
                                CFieldValue ttNValue = new CFieldValue();
                                ttNValue.value = ttRunSet;
                                CFieldValue[] ttNFValues = ttFValues;
                                if (ttNFValues.Length > 0)
                                {
                                    ttNFValues.SetValue(ttNValue, ttNFValues.Length - 1);
                                }
                                else
                                {
                                    ttNFValues.SetValue(ttNValue, 0);
                                }
                                try
                                {
                                    ttSoap.addDropdownFieldValuesForTable(ttCookie, "Test Run", "Test Run Set", ttNFValues);
                                }
                                catch (Exception fcc)
                                {
                                    Logger.WriteLine("UNable to add Test Run Set. " + fcc.Message);
                                }
                            }
                            //Define the columns we need to have in the list of test cases.  We only need the number
                            CTableColumn[] ttColumns = new CTableColumn[1];
                            ttColumns[0] = new CTableColumn();
                            ttColumns[0].name = "Number";
                            CRecordListSoap ttList = ttSoap.getRecordListForTable(ttCookie, "Test Case", ttFilter, ttColumns);
                            CRecordRowSoap[] ttRecords = ttList.records;
                            //For each test case
                            for (int j = 0; j < ttRecords.Length; j++)
                            {
                                tcNumber = 0;
                                tcNumber = Convert.ToInt64(ttRecords[j].row[0].value);
                                CTestCase ttTCase = ttSoap.getTestCase(ttCookie, tcNumber, "", false);
                                //before we can generate the test run for the test case, we need
                                //to build our test runs.  We'll use all variants defined in the test case
                                CTestRunVariantField[] ttRunVariants = new CTestRunVariantField[ttTCase.testVariants.Length];
                                for (int k = 0; k < ttTCase.testVariants.Length; k++)
                                {
                                    //Need to make sure we only look at the "Included" variants, and not the "Excluded" ones.
                                    if (ttTCase.testVariants[k].type == "Included")
                                    {
                                        CFieldValue[] tvVal = new CFieldValue[ttTCase.testVariants[k].values.Length];
                                        //Go through each value in this variant 
                                        for (int l = 0; l < ttTCase.testVariants[k].values.Length; l++)
                                        {
                                            tvVal[l] = new CFieldValue();
                                            tvVal[l].value = ttTCase.testVariants[k].values[l].value;
                                        }
                                        ttRunVariants[k] = new CTestRunVariantField();
                                        ttRunVariants[k].name = ttTCase.testVariants[k].name;
                                        ttRunVariants[k].values = tvVal;
                                    }
                                }
                                //Now we can generate the test run 
                                CItemToTrack[] ttRuns = ttSoap.generateTestRuns(ttCookie, ttTCase.recordid, ttRunVariants, ttRunSet);

                            }
                            //code to get list of test cases

                        }
                        catch (Exception fcc)
                        {
                            Logger.WriteLine("General Exception occurred. " + fcc.Message);
                        }
                    }
                    else
                    {
                        Logger.WriteLine("Problem configuring the project");
                    }
                }
            }
            else
            {
                Logger.WriteLine("Incorrect number of parameters were passed");
            }
        }
    }
}
